# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Bgp_global parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def _tmplt_confederation_peers(config_data):
    cmds = []
    base_cmd = "bgp confederation peers "
    peers = config_data.get("bgp", {}).get("confederation", {}).get("peers")
    if peers:
        for peer in peers:
            cmds.append(base_cmd + str(peer))
    return cmds


def _templ_local_as(config_data):
    conf = config_data.get("local_as", {})
    command = ""
    if conf.get("value"):
        command = "local-as " + str(conf.get("value", {}))
    if "no_prepend" in conf:
        if "replace_as" in conf.get("no_prepend", {}):
            if "dual_as" in conf.get("no_prepend", {}).get("replace_as", {}):
                command += " no-prepend replace-as dual-as"
            elif "set" in conf.get("no_prepend", {}).get("replace_as", {}):
                command += " no-prepend replace-as"
        elif "set" in conf.get("no_prepend", {}):
            command += " no-prepend"
    return command


class Bgp_globalTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Bgp_globalTemplate, self).__init__(
            lines=lines,
            tmplt=self,
            module=module,
        )

    # fmt: off
    PARSERS = [
        {
            "name": "router",
            "getval": re.compile(
                r"""
                ^router\s
                bgp
                \s(?P<as_num>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "router bgp {{ as_number }}",
            "compval": "as_number",
            "result": {"as_number": "{{ as_num }}"},
            "shared": True,
        },
        {
            "name": "vrf",
            "getval": re.compile(
                r"""
                \s+vrf
                \s(?P<vrf>\S+)$""",
                re.VERBOSE,
            ),
            "setval": "vrf {{ vrf }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "vrf": "{{ vrf }}",
                    },
                },
            },
            "shared": True,
        },

        {
            "name": "bfd_minimum_interval",
            "getval": re.compile(
                r"""
                \s+bfd\s(?P<min_interval>minimum-interval\s\d+)
                $""", re.VERBOSE,
            ),
            "compval": "bfd.minimum_interval",
            "setval": "bfd minimum-interval {{bfd.minimum_interval}}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bfd": {"minimum_interval": "{{ min_interval.split(" ")[1] }}"},
                    },
                },
            },
        },
        {
            "name": "bfd_multiplier",
            "getval": re.compile(
                r"""
                \s+bfd\s(?P<multiplier>multiplier\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "bfd multiplier {{bfd.multiplier}}",
            "compval": "bfd.multiplier",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bfd": {"multiplier": "{{multiplier.split(" ")[1]}}"},
                    },
                },
            },
        },
        {
            "name": "bgp_as_path_loopcheck",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<loopcheck>as-path-loopcheck)
                $""", re.VERBOSE,
            ),
            "setval": "bgp as-path-loopcheck",
            "compval": "bgp.as_path_loopcheck",
            "result": {
                "bgp": {
                    "as_path_loopcheck": "{{ True if loopcheck is defined }}",
                },
            },
        },
        {
            "name": "bgp_auto_policy_soft_reset",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<auto_policy_soft_reset_disable>auto-policy-soft-reset\sdisable)
                $""", re.VERBOSE,
            ),
            "setval": "bgp auto-policy-soft-reset disable",
            "compval": "bgp.auto_policy_soft_reset",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "auto_policy_soft_reset": {
                                "disable": "{{True if auto_policy_soft_reset_disable is defined}}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_cluster_id",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<cluster_id>cluster-id\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "bgp cluster-id {{bgp.cluster_id}}",
            "compval": "bgp.cluster_id",
            "result": {
                "bgp": {
                    "cluster_id": "{{cluster_id.split(" ")[1]}}",
                },
            },
        },
        {
            "name": "bgp_default_local_preference",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<default_local_pref>default\slocal-preference\s\S+)
                $""", re.VERBOSE,
            ),
            "setval": "bgp default local-preference {{bgp.default.local_preference}}",
            "compval": "bgp.default.local-preference",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "default": {
                                "local_preference": "{{default_local_pref.split(" ")[2] }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_enforce_first_as_disable",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<enforce_first_as_disable>enforce-first-as\sdisable)
                $""", re.VERBOSE,
            ),
            "setval": "bgp enforce-first-as disable",
            "compval": "bgp.enforce_first_as.disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "enforce_first_as": {
                                "disable": "{{ True if enforce_first_as_disable is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_fast_external_fallover_disable",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<fast_external_fallover_disable>fast-external-fallover\sdisable)
                $""", re.VERBOSE,
            ),
            "setval": "bgp fast-external-fallover disable",
            "compval": "bgp.fast_external_fallover.disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "fast_external_fallover": {
                                "disable": "{{True if fast_external_fallover_disable is defined}}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_install_diversion",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<install_diversion>install\sdiversion)
                $""", re.VERBOSE,
            ),
            "setval": "bgp install diversion",
            "compval": "bgp.install.diversion",
            "result": {
                "bgp": {
                    "install": {
                        "diversion": "{{True if install_diversion is defined}}",
                    },
                },
            },
        },
        {
            "name": "bgp_max_neighbors",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<max_neighbors>maximum\sneighbor\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "bgp maximum neighbor {{bgp.maximum.neighbor}}",
            "compval": "bgp.maximum.neighbor",
            "result": {
                "bgp": {
                    "maximum":
                        {
                            "neighbor": "{{max_neighbors.split(" ")[2] }}",
                        },
                },
            },
        },
        {
            "name": "bgp_redistribute_internal",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<redistribute_internal>redistribute-internal)
                $""", re.VERBOSE,
            ),
            "setval": "bgp redistribute-internal",
            "compval": "bgp.redistribute_internal",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "redistribute_internal": "{{ True if redistribute_internal is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_router_id",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<router_id>router-id\s\S+)
                $""", re.VERBOSE,
            ),
            "setval": "bgp router-id {{ bgp.router_id }}",
            "compval": "bgp.router_id",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "router_id": "{{router_id.split(" ")[1]}}",
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_scan_time",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<scan_time>scan-time\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "bgp scan-time {{ bgp.scan_time }}",
            "compval": "bgp.scan_time",
            "result": {
                "bgp": {
                    "scan_time": "{{scan_time.split(" ")[1]}}",
                },
            },
        },
        {
            "name": "bgp_unsafe_ebgp_policy",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<unsafe_ebgp_policy>unsafe-ebgp-policy)
                $""", re.VERBOSE,
            ),
            "setval": "bgp unsafe-ebgp-policy",
            "compval": "bgp.unsafe_ebgp_policy",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "unsafe_ebgp_policy": "{{ True if unsafe_ebgp_policy is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_update_delay",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<update_delay>update-delay\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "bgp update-delay {{ bgp.update_delay }}",
            "compval": "bgp.update_delay",
            "result": {
                "bgp": {
                    "update_delay": "{{update_delay.split(" ")[1]}}",

                },
            },
        },
        {
            "name": "bgp_bestpath_aigp",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<bestpath_aigp_ignore>bestpath\saigp\signore)
                $""", re.VERBOSE,
            ),
            "setval": "bgp bestpath aigp ignore",
            "compval": "bgp.bestpath.aigp.ignore",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "bestpath": {
                                "aigp": {
                                    "ignore": "{{ True if bestpath_aigp_ignore is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_bestpath_as_path_ignore",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<as_path_ignore>bestpath\sas-path\signore)
                $""", re.VERBOSE,
            ),
            "setval": "bgp bestpath as-path ignore",
            "compval": "bgp.bestpath.as_path.ignore",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "bestpath": {
                                "as_path": {
                                    "ignore": "{{ True if as_path_ignore is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_bestpath_as_path_multipath_relax",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<as_path_multipath_relax>bestpath\sas-path\smultipath-relax)
                $""", re.VERBOSE,
            ),
            "setval": "bgp bestpath as-path multipath-relax",
            "compval": "bgp.bestpath.as_path.multipath_relax",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "bestpath": {
                                "as_path": {
                                    "multipath_relax": "{{ True if as_path_multipath_relax is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_bestpath_med_always",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<always>bestpath\smed\salways)
                $""", re.VERBOSE,
            ),
            "setval": "bgp bestpath med always",
            "compval": "bgp.bestpath.med.always",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "bestpath": {
                                "med": {
                                    "always": "{{ True if always is defined}}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_bestpath_med_confed",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<confed>bestpath\smed\sconfed)
                $""", re.VERBOSE,
            ),
            "setval": "bgp bestpath med confed",
            "compval": "bgp.bestpath.med.confed",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "bestpath": {
                                "med": {
                                    "confed": "{{ True if confed is defined}}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_bestpath_med_missing_as_worst",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<missing_as_worst>bestpath\smed\smissing-as-worst)
                $""", re.VERBOSE,
            ),
            "setval": "bgp bestpath med missing-as-worst)",
            "compval": "bgp.bestpath.med.missing_as_worst",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "bestpath": {
                                "med": {
                                    "missing_as_worst": "{{ True if missing_as_worst is defined}}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_bestpath_compare_routerid",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<compare_routerid>bestpath\scompare-routerid)
                $""", re.VERBOSE,
            ),
            "setval": "bgp bestpath compare-routerid",
            "compval": "bgp.bestpath.compare_routerid",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "bestpath": {
                                "compare_routerid": "{{ True if compare_routerid is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_bestpath_cost_community_ignore",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<cost_community_ignore>bestpath\scost-community\signore)
                $""", re.VERBOSE,
            ),
            "setval": "bgp bestpath cost-community ignore",
            "compval": "bgp.bestpath.cost_community.ignore",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "bestpath": {
                                "cost_community": {
                                    "ignore": "{{ True if cost_community_ignore is defined}}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_bestpath_origin_as_use",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<origin_as_use>bestpath\sorigin-as\suse\svalidity)
                $""", re.VERBOSE,
            ),
            "setval": "bgp bestpath origin-as use validity",
            "compval": "bgp.bestpath.origin_as.use.validity",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "bestpath": {
                                "origin_as": {
                                    "use": {"validity": "{{ True if origin_as_use is defined }}"},
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_bestpath_origin_as_allow",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<origin_as_allow>bestpath\sorigin-as\sallow\sinvalid)
                $""", re.VERBOSE,
            ),
            "setval": "bgp bestpath origin-as allow invalid",
            "compval": "bgp.bestpath.origin_as.allow.invalid",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "bestpath": {
                                "origin_as":
                                    {
                                        "allow": {"invalid": "{{ True if origin_as_allow is defined }}"},
                                    },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_confederation_identifier",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<confederation_identifier>confederation\sidentifier\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "bgp confederation identifier {{ bgp.confederation.identifier}}",
            "compval": "bgp.confederation.identifier",
            "result": {
                "bgp": {
                    "confederation": {
                        "identifier": "{{confederation_identifier.split(" ")[2]}}",
                    },

                },
            },
        },
        {
            "name": "bgp_confederation_peers",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<confederation_peers>confederation\speers\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": _tmplt_confederation_peers,
            "compval": "bgp.confederation.peers",
            "result": {
                "bgp": {
                    "confederation": {
                        "peers": {
                            "peer" + "{{confederation_peers.split(" ")[2]}}": "{{confederation_peers.split(" ")[2]}}",
                        },
                    },

                },
            },
        },
        {
            "name": "bgp_graceful_restart_set",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<graceful_restart_set>graceful-restart)
                $""", re.VERBOSE,
            ),
            "setval": "bgp graceful-restart",
            "compval": "bgp.graceful_restart.set",
            "result": {
                "bgp": {
                    "graceful_restart": {
                        "set": "{{ True if graceful_restart_set is defined }}",
                    },

                },
            },
        },
        {
            "name": "bgp_graceful_restart_graceful_reset",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<graceful_restart_graceful_reset>graceful-restart\sgraceful-reset)
                $""", re.VERBOSE,
            ),
            "setval": "bgp graceful-restart graceful-reset",
            "compval": "bgp.graceful_restart.graceful_reset",
            "result": {
                "bgp": {
                    "graceful_restart": {
                        "graceful_reset": "{{ True if graceful_restart_graceful_reset is defined}}",
                    },

                },
            },
        },
        {
            "name": "bgp_graceful_restart_restart_time",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<graceful_restart_restart_time>graceful-restart\srestart-time\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "bgp graceful-restart restart-time {{ bgp.graceful_restart.restart_time}}",
            "compval": "bgp.graceful_restart.restart_time",
            "result": {
                "bgp": {
                    "graceful_restart": {
                        "restart_time": "{{ graceful_restart_restart_time.split(" ")[2] }}",
                    },

                },
            },
        },
        {
            "name": "bgp_graceful_restart_purge_time",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<graceful_restart_purge_time>graceful-restart\spurge-time\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "bgp graceful-restart purge-time {{ bgp.graceful_restart.purge_time}}",
            "compval": "bgp.graceful_restart.purge_time",
            "result": {
                "bgp": {
                    "graceful_restart": {
                        "purge_time": "{{ graceful_restart_purge_time.split(" ")[2] }}",
                    },

                },
            },
        },
        {
            "name": "bgp_graceful_restart_stalepath_time",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<graceful_restart_stalepath_time>graceful-restart\sstalepath-time\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "bgp graceful-restart stalepath-time {{ bgp.graceful_restart.stalepath_time}}",
            "compval": "bgp.graceful_restart.stalepath_time",
            "result": {
                "bgp": {
                    "graceful_restart": {
                        "stalepath_time": "{{ graceful_restart_stalepath_time.split(" ")[2] }}",
                    },

                },
            },
        },
        {
            "name": "bgp_log_message",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<log_message>log\smessage\sdisable)
                $""", re.VERBOSE,
            ),
            "setval": "bgp log message disable",
            "compval": "bgp.log_message.message.disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "log": {
                                "log_message": {"disable": "{{ True if log_message is defined }}"},
                            },
                        },
                    },

                },
            },
        },
        {
            "name": "bgp_log_neighbor_changes_detail",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<log_neighbor_changes_detail>log\sneighbor\schanges\sdetail)
                $""", re.VERBOSE,
            ),
            "setval": "bgp log neighbor changes detail",
            "compval": "bgp.log.neighbor.changes.detail",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "log": {
                                "neighbor": {
                                    "changes": {
                                        "detail": "{{True if log_neighbor_changes_detail is defined }}",
                                    },
                                },
                            },
                        },
                    },

                },
            },
        },
        {
            "name": "bgp_log_neighbor_changes_disable",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<log_neighbor_changes_disable>log\sneighbor\schanges\sdisable)
                $""", re.VERBOSE,
            ),
            "setval": "bgp log neighbor changes disable",
            "compval": "bgp.log.neighbor.changes.disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "log": {
                                "neighbor": {
                                    "changes": {
                                        "disable":
                                            "{{ True if log_neighbor_changes_disable is defined }}",
                                    },
                                },
                            },
                        },
                    },

                },
            },
        },
        {
            "name": "bgp_multipath_as_path_ignore_onwards",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<multipath>multipath\sas-path\signore\sonwards)
                $""", re.VERBOSE,
            ),
            "setval": "bgp multipath as-path ignore onwards",
            "compval": "bgp.multipath.as_path.ignore.onwards",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "bgp": {
                            "multipath": {
                                "as_path": {"ignore": {"onwards": "{{ not not multipath}}"}},
                            },
                        },
                    },

                },
            },
        },
        {
            "name": "bgp_origin_as_validation_disable",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<origin_as_validation_disable>origin-as\svalidation\sdisable)
                $""", re.VERBOSE,
            ),
            "setval": "bgp origin-as validation disable",
            "compval": "bgp.origin_as.validation.disable",
            "result": {
                "bgp": {
                    "origin_as": {
                        "validation": {"disable": "{{ not not origin_as_validation_disable}}"},
                    },

                },
            },
        },
        {
            "name": "bgp_origin_as_validation_signal_ibgp",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<origin_as_validation_signal_ibgp>origin-as\svalidation\ssignal\sibgp)
                $""", re.VERBOSE,
            ),
            "setval": "bgp origin-as validation signal ibgp",
            "compval": "bgp.origin_as.validation.signal.ibgp",
            "result": {
                "bgp": {
                    "origin_as": {
                        "validation": {"signal": {"ibgp": "{{ not not origin_as_validation_signal_ibgp }}"}},
                    },

                },
            },
        },
        {
            "name": "bgp_origin_as_validation_time_off",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<validation_time_off>origin-as\svalidation\stime\soff)
                $""", re.VERBOSE,
            ),
            "setval": "bgp origin-as validation time off",
            "compval": "bgp.origin_as.validation.time.off",
            "result": {
                "bgp": {
                    "origin_as": {
                        "validation": {"time": {"time_off": "{{ not not validation_time_off }}"}},
                    },

                },
            },
        },
        {
            "name": "bgp_origin_as_validation_time",
            "getval": re.compile(
                r"""
                \s+bgp\s(?P<validation_time>origin-as\svalidation\stime\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "bgp origin-as validation time {{ bgp.origin_as.validation.time.time_in_second }}",
            "compval": "bgp.origin_as.validation.time.time_in_second",
            "result": {
                "bgp": {
                    "origin_as": {
                        "validation": {"time": {"time_in_second": "{{ validation_time.split(" ")[3] }}"}},
                    },

                },
            },
        },

        {
            "name": "bgp_default_information_originate",
            "getval": re.compile(
                r"""
                \s+default-information\s(?P<default_information_originate>originate)
                $""", re.VERBOSE,
            ),
            "setval": "default-information originate",
            "compval": "default_information.originate",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "default_information": {
                            "originate": "{{ not not default_information_originate }}",
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_default_metric",
            "getval": re.compile(
                r"""
                \s+default-metric\s(?P<default_metric>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "default-metric {{default_metric}}",
            "compval": "default_metric",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "default_metric": "{{ default_metric }}",
                    },
                },
            },
        },
        {
            "name": "bgp_graceful_maintenance",
            "getval": re.compile(
                r"""
                \s+graceful-maintenance\sactivate\s(?P<graceful_maintenance>\S*)
                $""", re.VERBOSE,
            ),
            "setval": "graceful_maintenance {{graceful_maintenance.activate}}",
            "compval": "graceful_maintenance.activate",
            "result": {
                "graceful_maintenance": {"activate": "{{ graceful_maintenance }}"},
            },
        },
        {
            "name": "ibgp_policy_out_enforce_modifications",
            "getval": re.compile(
                r"""
                \s+ibgp\spolicy\sout\s(?P<ibgp_policy_out>enforce-modifications)
                $""", re.VERBOSE,
            ),
            "setval": "ibgp policy out enforce-modifications",
            "compval": "ibgp.policy.out.enforce_modifications",
            "result": {
                "ibgp": {"policy": {"out": {"enforce_modifications": "{{ not not ibgp_policy_out }}"}}},
            },
        },
        {
            "name": "mpls_activate_interface",
            "getval": re.compile(
                r"""
                \s+mpls\sactivate\sinterface(?P<mpls_interface>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "mpls activate interface {{mpls.activate.interface}}",
            "compval": "mpls.activate.interface",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "mpls": {"activate": {"interface": "{{ mpls_interface }}"}},
                    },
                },
            },
        },
        {
            "name": "mvpn",
            "getval": re.compile(
                r"""
                \s(?P<mvpn>mvpn)
                $""", re.VERBOSE,
            ),
            "setval": "mvpn",
            "compval": "mvpn",
            "result": {
                "mvpn": "{{ not not mvpn }}",
            },
        },
        {
            "name": "nsr_set",
            "getval": re.compile(
                r"""
                \s(?P<nsr>nsr\s*)
                $""", re.VERBOSE,
            ),
            "setval": "nsr",
            "compval": "nsr.set",
            "result": {
                "nsr": {"set": "{{ not not nsr }}"},
            },
        },
        {
            "name": "nsr_disable",
            "getval": re.compile(
                r"""
                \snsr\s(?P<nsr_disable>disable\s*)
                $""", re.VERBOSE,
            ),
            "setval": "nsr disable",
            "compval": "nsr.disable",
            "result": {
                "nsr": {"disable": "{{ not not nsr_disable }}"},
            },
        },
        {
            "name": "socket_receive_buffer_size",
            "getval": re.compile(
                r"""
                \s+socket\s(?P<socket_rcv_buffer_size>receive-buffer-size\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "socket receive-buffer-size {{ socket.receive_buffer_size}}",
            "compval": "socket.receive_buffer_size",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "socket": {"receive_buffer_size": "{{ socket_rcv_buffer_size.split(" ")[1] }}"},
                    },
                },
            },
        },
        {
            "name": "socket_send_buffer_size",
            "getval": re.compile(
                r"""
                \s+socket\s(?P<socket_send_buffer_size>send-buffer-size\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "socket send-buffer-size {{ socket.send_buffer_size}}",
            "compval": "socket.send_buffer_size",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "socket": {"send_buffer_size": "{{ socket_send_buffer_size.split(" ")[1] }}"},
                    },
                },
            },
        },
        {
            "name": "update_in_error_handling_basic_ebgp_disable",
            "getval": re.compile(
                r"""
                \s+update\sin\serror-handling\sbasic\sebgp\s(?P<disable>disable)
                $""", re.VERBOSE,
            ),
            "setval": "update in error-handling basic ebgp disable",
            "compval": "update.in.error_handling.basic.ebgp.disable",
            "result": {
                "update": {"in": {"error_handling": {"basic": {"ebgp": {"disable": "{{ not not disable }}"}}}}},
            },
        },
        {
            "name": "update_in_error_handling_basic_ibgp_disable",
            "getval": re.compile(
                r"""
                \s+update\sin\serror-handling\sbasic\sibgp\s(?P<disable>disable)
                $""", re.VERBOSE,
            ),
            "setval": "update in error-handling basic ibgp disable",
            "compval": "update.in.error_handling.basic.ibgp.disable",
            "result": {
                "update": {"in": {"error_handling": {"basic": {"ibgp": {"disable": "{{ not not disable }}"}}}}},
            },
        },
        {
            "name": "update_in_error_handling_extended_ebgp",
            "getval": re.compile(
                r"""
                \s+update\sin\serror-handling\sextended\s(?P<extended_ebgp>ebgp)
                $""", re.VERBOSE,
            ),
            "setval": "update in error-handling extended ebgp",
            "compval": "update.in.error_handling.extended.ebgp",
            "result": {
                "update": {"in": {"error_handling": {"extended": {"ebgp": "{{ not not extended_ebgp}}"}}}},
            },
        },
        {
            "name": "update_in_error_handling_extended_ibgp",
            "getval": re.compile(
                r"""
                \s+update\sin\serror-handling\sextended\s(?P<extended_ibgp>ibgp)
                $""", re.VERBOSE,
            ),
            "setval": "update in error-handling extended ibgp",
            "compval": "update.in.error_handling.extended.ibgp",
            "result": {
                "update": {"in": {"error_handling": {"extended": {"ibgp": "{{ not not extended_ibgp}}"}}}},
            },
        },
        {
            "name": "update_out_logging",
            "getval": re.compile(
                r"""
                \s+update\sout\s(?P<update_out_logging>logging)
                $""", re.VERBOSE,
            ),
            "setval": "update out logging",
            "compval": "update.out.logging",
            "result": {
                "update": {"out": {"logging": "{{ not not update_out_logging}}"}},
            },
        },
        {
            "name": "update_limit",
            "getval": re.compile(
                r"""
                \s+update\slimit\s(?P<update_limit>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "update limit {{ update.limit }}",
            "compval": "update.limit",
            "result": {
                "update": {"limit": "{{ update_limit}}"},
            },
        },
        {
            "name": "rpki_route_value",
            "getval": re.compile(
                r"""
                \srpki
                \sroute
                \s(?P<value>\S+)
                \smax
                \s(?P<max>\d+)
                \sorigin
                \s(?P<origin>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "rpki route {{ rpki.route.value }} max {{rpki.route.max }} "
                      "origin {{rpki.route.origin }}",
            "compval": "rpki.route",
            "result": {
                "rpki": {
                    "route": {
                        "value": "{{value}}",
                        "origin": "{{origin}}",
                        "max": "{{max}}",
                    },
                },
            },
        },
        {
            "name": "rpki_server_name",
            "getval": re.compile(
                r"""
                \srpki
                \s(?P<value>server\s\S+)
                $""", re.VERBOSE,
            ),
            "setval": "rpki server {{ name }}",
            "compval": "rpki.server.name",
            "result": {
                "rpki":
                    {
                        "servers": {"{{value.split(" ")[1]}}": {"name": "{{ value.split(" ")[1] }}"}},
                    },
            },
        },
        {
            "name": "rpki_server_purge_time",
            "getval": re.compile(
                r"""
                \srpki
                \s(?P<rpki_server>server\s\S+)
                \s(?P<purge_time>purge-time\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "purge-time {{ purge_time }}",
            "compval": "purge_time",
            "result": {
                "rpki":
                    {
                        "servers": {"{{rpki_server.split(" ")[1]}}": {"purge_time": "{{ purge_time.split(" ")[1] }}"}},
                    },
            },
        },
        {
            "name": "rpki_server_refresh_time",
            "getval": re.compile(
                r"""
                \srpki
                \s(?P<rpki_server>server\s\S+)
                \s(?P<refresh_time>refresh-time\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "refresh-time {{ refresh_time.value }}",
            "compval": "refresh_time.value",
            "result": {
                "rpki": {
                    "servers": {
                        "{{rpki_server.split(" ")[1]}}": {
                            "refresh_time": {
                                "value": "{{ refresh_time.split(" ")[1] }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "rpki_server_refresh_time_off",
            "getval": re.compile(
                r"""
                \srpki
                \s(?P<rpki_server>server\s\S+)
                \srefresh-time
                \s(?P<refresh_time>off)
                $""", re.VERBOSE,
            ),
            "setval": "refresh-time off",
            "compval": "refresh_time.time_off",
            "result": {
                "rpki": {
                    "servers": {
                        "{{rpki_server.split(" ")[1] }}": {
                            "refresh_time": {
                                "time_off": "{{ True if refresh_time is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "rpki_server_response_time_off",
            "getval": re.compile(
                r"""
                \srpki
                \s(?P<rpki_server>server\s\S+)
                \sresponse-time
                \s(?P<response_time>off)
                $""", re.VERBOSE,
            ),
            "setval": "response-time off",
            "compval": "response_time.time_off",
            "result": {
                "rpki": {
                    "servers": {
                        "{{rpki_server.split(" ")[1]}}": {
                            "response_time": {
                                "time_off": "{{ True if response_time is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "rpki_server_response_time",
            "getval": re.compile(
                r"""
                \srpki
                \s(?P<rpki_server>\sserver\S+)
                \s(?P<response_time>response-time\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "response-time {{ response_time.value }}",
            "compval": "response_time.value",
            "result": {
                "rpki": {
                    "servers": {
                        "{{rpki_server.split(" ")[1]}}": {
                            "response_time": {
                                "value": "{{ response_time.split(" ")[1] }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "rpki_server_shutdown",
            "getval": re.compile(
                r"""
                \srpki
                \s(?P<rpki_server>server\s\S+)
                \s(?P<shutdown>shutdown)
                $""", re.VERBOSE,
            ),
            "setval": "shutdown",
            "compval": "shutdown",
            "result": {
                "rpki": {
                    "servers": {"{{rpki_server.split(" ")[1]}}": {"shutdown": "{{ True if shutdown  is defined}}"}},
                },
            },
        },
        {
            "name": "rpki_server_transport_ssh",
            "getval": re.compile(
                r"""
                \srpki
                \s(?P<rpki_server>server\s\S+)
                \stransport
                \sssh
                \s(?P<ssh_port>port\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "transport ssh port {{ transport.ssh.port }}",
            "compval": "transport.ssh.port",
            "result": {
                "rpki": {
                    "servers": {
                        "{{rpki_server.split(" ")[1]}}": {
                            "transport": {
                                "ssh": {
                                    "port": "{{ ssh_port.split(" ")[1] }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "rpki_server_transport_tcp",
            "getval": re.compile(
                r"""
                \srpki
                \s(?P<rpki_server>server\s\S+)
                \stransport
                \stcp
                \s(?P<tcp_port>port\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "transport tcp port {{ transport.tcp.port }}",
            "compval": "transport.tcp.port",
            "result": {
                "rpki": {
                    "servers": {
                        "{{rpki_server.split(" ")[1]}}": {
                            "transport": {
                                "tcp": {
                                    "port": "{{ tcp_port.split(" ")[1] }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_address",
            "getval": re.compile(
                r"""
                \s+neighbor\s(?P<value>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "neighbor {{ neighbor_address }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{value}}":
                                {
                                    "neighbor_address": "{{value}}",
                                },
                        },
                    },
                },
            },
        },
        {
            "name": "advertisement_interval",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<advertise_in>advertisement-interval\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "advertisement-interval {{ advertisement_interval }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "advertisement_interval": "{{ advertise_in.split(" ")[1] }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bfd_fast_detect_disable",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \sbfd
                \sfast-detect
                \s(?P<disable>disable)
                $""", re.VERBOSE,
            ),
            "setval": "bfd fast-detect disable",
            "compval": "bfd.fast_detect.disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "bfd": {
                                    "fast_detect": {"disable": "{{ True if disable is defined }}"},
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bfd_fast_detect_set",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \sbfd
                \s(?P<fast_detect>fast-detect)
                $""", re.VERBOSE,
            ),
            "setval": "bfd fast-detect",
            "compval": "bfd.fast_detect.set",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "bfd": {
                                    "fast_detect": {"set": "{{ True if fast_detect is defined }}"},
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bfd_fast_detect_strict_mode",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \sbfd
                \sfast-detect
                \s(?P<strict_mode>strict-mode)
                $""", re.VERBOSE,
            ),
            "setval": "bfd fast-detect strict-mode",
            "compval": "bfd.fast_detect.strict_mode",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "bfd": {
                                    "fast_detect": {"strict_mode": "{{ True if strict_mode is defined }}"},
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bfd_nbr_multiplier",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \sbfd
                \s(?P<multiplier>multiplier\s\S+)
                $""", re.VERBOSE,
            ),
            "setval": "bfd multiplier {{ bfd.multiplier}}",
            "compval": "bfd.multiplier",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}":
                                {
                                    "bfd":
                                    {
                                        "multiplier": "{{multiplier.split(" ")[1]}}",
                                    },
                                },
                        },
                    },
                },
            },
        },
        {
            "name": "bfd_nbr_minimum_interval",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \sbfd
                \s(?P<min_interval>minimum-interval\s\S+)
                $""", re.VERBOSE,
            ),
            "setval": "bfd minimum-interval {{ bfd.minimum_interval}}",
            "compval": "bfd.minimum_interval",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}":
                                {
                                    "bfd":
                                    {
                                        "minimum_interval": "{{min_interval.split(" ")[1]}}",
                                    },
                                },
                        },
                    },
                },
            },
        },
        {
            "name": "bmp_activate",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \sbmp-activate
                \s(?P<bmp_activate>server\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "bmp-activate server {{bmp_activate.server}}",
            "compval": "bmp_activate.serevr",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "bmp_activate": {"server": "{{ bmp_activate.split(" ")[1] }}"},
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_cluster_id",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<cluster_id>cluster-id\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "cluster-id {{ cluster_id }}",
            "compval": "cluster_id",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {"cluster_id": "{{ cluster_id.split(" ")[1] }}"},
                        },
                    },
                },

            },
        },
        {
            "name": "neighbor_description",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \sdescription\s(?P<description>.+)
                $""", re.VERBOSE,
            ),
            "setval": "description {{ description }}",
            "compval": "description",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {"description": "{{ description }}"},
                        },
                    },
                },
            },
        },
        {
            "name": "dmz_link_bandwidth",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<dmz_link_bandwidth>dmz-link-bandwidth)
                $""", re.VERBOSE,
            ),
            "setval": "dmz-link-bandwidth",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}":
                                {
                                    "dmz_link_bandwidth":
                                    {
                                        "set": "{{ True if dmz_link_bandwidth is defined }}",
                                    },
                                },
                        },
                    },
                },
            },
        },
        {
            "name": "dmz_link_bandwidth_inheritance_disable",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \sdmz-link-bandwidth
                \s(?P<dmz_link_bandwidth>inheritance_disable)
                $""", re.VERBOSE,
            ),
            "setval": "dmz-link-bandwidth inheritance-disable",
            "compval": "dmz_link_bandwidth.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "dmz_link_bandwidth":
                                {
                                    "inheritance_disable": "{{ True if dmz_link_bandwidth is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "ebgp_multihop_value",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<ebgp_multihop>ebgp-multihop\s\S+)
                $""", re.VERBOSE,
            ),
            "setval": "ebgp-multihop {{ ebgp_multihop.value}}",
            "compval": "ebgp_multihop.value",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "ebgp_multihop": {
                                    "value": "{{ ebgp_multihop.split(" ")[1] }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "ebgp_multihop_mpls",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<ebgp_multihop>ebgp-multihop\s\S*\smpls)
                $""", re.VERBOSE,
            ),
            "setval": "ebgp-multihop mpls",
            "compval": "ebgp_multihop.mpls",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "ebgp_multihop": {"mpls": "{{ True if ebgp_multihop is defined }}"},
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "ebgp_recv_extcommunity_dmz",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<ebgp_recv_extcommunity_dmz>ebgp-recv-extcommunity-dmz\sinheritance-disable)
                $""", re.VERBOSE,
            ),
            "setval": "ebgp-recv-extcommunity-dmz inheritance-disable ",
            "compval": "ebgp_recv_extcommunity_dmz.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "ebgp_recv_extcommunity_dmz": {
                                    "inheritance_disable": "{{ True if ebgp_recv_extcommunity_dmz is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "ebgp_recv_extcommunity_dmz_set",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<ebgp_recv_extcommunity_dmz>ebgp-recv-extcommunity-dmz)
                $""", re.VERBOSE,
            ),
            "setval": "ebgp-recv-extcommunity-dmz inheritance-disable",
            "compval": "ebgp_recv_extcommunity_dm.set",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "ebgp_recv_extcommunity_dmz": {
                                    "set": "{{ True if ebgp_recv_extcommunity_dmz is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "ebgp_send_extcommunity_dmz",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<ebgp_send_extcommunity_dmz>ebgp-send-extcommunity-dmz\sinheritance-disable)
                $""", re.VERBOSE,
            ),
            "setval": "ebgp-send-extcommunity-dmz inheritance-disable ",
            "compval": "ebgp_send_extcommunity_dmz.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "ebgp_send_extcommunity_dmz": {
                                    "inheritance_disable": "{{ True if ebgp_send_extcommunity_dmz is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "ebgp_send_extcommunity_dmz_set",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<ebgp_send_extcommunity_dmz>ebgp-send-extcommunity-dmz)
                $""", re.VERBOSE,
            ),
            "setval": "ebgp-send-extcommunity-dmz",
            "compval": "ebgp_send_extcommunity_dmz.set",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "ebgp_send_extcommunity_dmz": {
                                    "set": "{{ True if ebgp_send_extcommunity_dmz is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "ebgp_send_extcommunity_dmz_cumulatie",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<ebgp_send_extcommunity_dmz>ebgp-send-extcommunity-dmz\scumulatie)
                $""", re.VERBOSE,
            ),
            "setval": "ebgp-send-extcommunity-dmz cumulatie ",
            "compval": "ebgp_send_extcommunity_dmz.cumulatie",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "ebgp_send_extcommunity_dmz": {
                                    "cumulatie": "{{ True if ebgp_send_extcommunity_dmz is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "egress_engineering",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<egress_engineering>egress-engineering\sinheritance-disable)
                $""", re.VERBOSE,
            ),
            "setval": "egress-engineering inheritance-disable ",
            "compval": "egress_engineering.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "egress_engineering": {
                                    "inheritance_disable": "{{ True if egress_engineering is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "egress_engineering_set",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<egress_engineering>egress-engineering)
                $""", re.VERBOSE,
            ),
            "setval": "egress-engineering",
            "compval": "egress_engineering.set",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "egress_engineering": {
                                    "set": "{{ True if egress_engineering is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_enforce_first_as_disable",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<enforce_first_as_disable>enforce-first-as\sdisable)
                $""", re.VERBOSE,
            ),
            "setval": "enforce-first-as disable",
            "compval": "enforce_first_as.disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "enforce_first_as": {
                                    "disable": "{{ True if enforce_first_as_disable is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_graceful_restart_restart_time",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<graceful_restart_restart_time>graceful-restart\srestart-time\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "graceful-restart restart-time {{ graceful_restart.restart_time}}",
            "compval": "graceful_restart.restart_time",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "graceful_restart": {
                                    "restart_time": "{{ graceful_restart_restart_time.split(" ")[2] }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_graceful_restart_stalepath_time",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<graceful_restart_stalepath_time>graceful-restart\sstalepath-time\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "graceful-restart stalepath-time {{ graceful_restart.stalepath_time}}",
            "compval": "graceful_restart.stalepath_time",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "graceful_restart": {
                                    "stalepath_time": "{{ graceful_restart_stalepath_time.split(" ")[2] }}",
                                },
                            },
                        },
                    },

                },
            },
        },
        {
            "name": "ignore_connected_check_set",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<ignore_connected_check>ignore-connected-check)
                $""", re.VERBOSE,
            ),
            "setval": "ignore-connected-check",
            "compval": "ignore_connected_check.set",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "ignore_connected_check": {
                                    "set": "{{ True if ignore_connected_check is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "ignore_connected_check",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<ignore_connected_check>ignore-connected-check\sinheritance-disable)
                $""", re.VERBOSE,
            ),
            "setval": "ignore-connected-check inheritance-disable ",
            "compval": "ignore_connected_check.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "ignore_connected_check": {
                                    "inheritance_disable": "{{ True if ignore_connected_check is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "keychain",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<keychain>keychain\sinheritance-disable)
                $""", re.VERBOSE,
            ),
            "setval": "keychain inheritance-disable ",
            "compval": "keychain.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "keychain": {
                                    "inheritance_disable": "{{ True if keychain is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "keychain_name",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<keychain>keychain\s\S+)
                $""", re.VERBOSE,
            ),
            "setval": "keychain {{ name }}",
            "compval": "keychain.name",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "keychain": {
                                    "name": "{{ keychain.split(" ")[1] }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "remote_as",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<remote_as>remote-as\s\S+)
                $""", re.VERBOSE,
            ),
            "setval": "remote-as {{ remote_as }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "remote_as": "{{ remote_as.split(" ")[1] }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "local_as_inheritance_disable",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<local_as>local-as\sinheritance-disable)
                $""", re.VERBOSE,
            ),
            "setval": "local-as inheritance-disable",
            "compval": "local_as.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "local_as": {
                                    "inheritance_disable": "{{ True if local_as is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "local_as",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<local_as>local-as\s\S+)
                (\s(?P<no_prepend>no-prepend))?
                (\s(?P<replace_as>replace-as))?
                (\s(?P<dual_as>dual-as))?
                $""", re.VERBOSE,
            ),
            "setval": _templ_local_as,
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "local_as": {
                                    "value": "{{ local_as.split(" ")[1] }}",
                                    "no_prepend":
                                        {
                                            "set": "{{ True if no_prepend is defined and replace_as is undefined and dual_as is undefined else None}}",
                                            "replace_as": {
                                                "set": "{{ True if replace_as is defined and dual_as is undefined}}",
                                                "dual_as": "{{ not not dual_as}}",
                                            },
                                        },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "local_address",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \slocal
                \s(?P<local>address\sinheritance-disable)
                $""", re.VERBOSE,
            ),
            "setval": "local address inheritance-disable",
            "compval": "local.address.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "local": {
                                    "address": {
                                        "inheritance_disable": "{{ True if local is defined }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "local",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \slocal
                \s(?P<local>address\s\S+)
                $""", re.VERBOSE,
            ),
            "setval": "local address {{ local.address.ipv4_address }}",
            "compval": "local.address.ipv4_address",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "local": {
                                    "address": {
                                        "ipv4_address": "{{ local.split(" ")[1] }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "origin_as",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \sorigin-as
                \s(?P<origin_as>validation\sdisable)
                $""", re.VERBOSE,
            ),
            "setval": "origin-as validation disable",
            "compval": "origin-as.validation.disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "origin_as": {
                                    "validation": {
                                        "disable": "{{ True if origin_as is defined}}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "password_inheritance_disable",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<password>password\sinheritance-disable)
                $""", re.VERBOSE,
            ),
            "setval": "password inheritance-disable",
            "compval": "password.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "password": {
                                    "inheritance_disable": "{{ True if password is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "password_encrypted",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \spassword\sencrypted
                \s(?P<password>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "password encrypted {{password.encrypted}}",
            "compval": "password.encrypted",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "password": {
                                    "encrypted": "{{ password }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "receive_buffer_size",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<receive_buffer_size>receive-buffer-size\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "receive-buffer-size {{ receive_buffer_size }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "receive_buffer_size": "{{ receive_buffer_size.split(" ")[1] }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "send_buffer_size",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<send_buffer_size>send-buffer-size\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "send-buffer-size {{ send_buffer_size }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "send_buffer_size": "{{ send_buffer_size.split(" ")[1] }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "session_open_mode",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<session_open_mode>session-open-mode\s(active-only|both|passive-only))
                $""", re.VERBOSE,
            ),
            "setval": "session-open-mode {{ session_open_mode }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "session_open_mode": "{{ session_open_mode.split(" ")[1] }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_shutdown",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<shutdown>shutdown)
                $""", re.VERBOSE,
            ),
            "setval": "shutdown",
            "compval": "shutdown",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "shutdown": {
                                    "set": "{{ True if shutdown is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_shutdown_inheritance_disable",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<shutdown>shutdown\sinheritance_disable)
                $""", re.VERBOSE,
            ),
            "setval": "shutdown inheritance-disable",
            "compval": "shutdown.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "shutdown": {"inheritance_disable": "{{ True if shutdown is defined }}"},
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "dscp",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<dscp>dscp\s\S+)
                $""", re.VERBOSE,
            ),
            "setval": "dscp {{ dscp }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "dscp": "{{ dscp.split(" ")[1] }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_tcp_mss_inheritance_disable",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<tcp_mss_disable>tcp\smss\sinheritance-disable)
                $""", re.VERBOSE,
            ),
            "setval": "tcp mss inheritance-disable",
            "compval": "tcp.mss.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "tcp": {
                                    "mss": {
                                        "inheritance_disable": "{{ True if tcp_mss_disable is defined }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_tcp_mss",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<tcp_mss>tcp\smss\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "tcp mss {{ tcp.mss.value }}",
            "compval": "tcp.mss.value",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "tcp": {
                                    "mss": {
                                        "value": "{{ tcp_mss.split(" ")[2] }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_timers_keepalive",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<timers_keepalive_time>timers\s\d+)
                \s(?P<timers_holdtime>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "timers {{ timers.keepalive_time}} {{ timers.holdtime }}",
            "compval": "timers",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "timers": {
                                    "keepalive_time": "{{ timers_keepalive_time.split(" ")[1] }}",
                                    "holdtime": "{{ timers_holdtime.split(" ")[0] }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "use.neighbor_group",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \suse\sneighbor-group\s(?P<neighbor_group>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "use neighbor-group {{ use.neighbor_group }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "use": {
                                    "neighbor_group": "{{ neighbor_group }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "use.session_group",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \suse\ssession-group\s(?P<session_group>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "use session-group {{ use.session_group }}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "use": {
                                    "session_group": "{{ session_group }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "update_source",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \supdate-source
                \s(?P<update_source>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "update-source {{ update_source}}",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "update_source": "{{ update_source}}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_ttl_security_inheritance_disable",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<ttl_security>ttl-security\sinheritance-disable)
                $""", re.VERBOSE,
            ),
            "setval": "ttl-security inheritance-disable",
            "compval": "ttl_security.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "ttl_security": {
                                    "inheritance_disable": "{{ True if ttl_security is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_ttl_security",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<ttl_security>ttl-security)
                $""", re.VERBOSE,
            ),
            "setval": "ttl-security",
            "compval": "ttl_security.set",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "ttl_security": {
                                    "set": "{{ True if ttl_security is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_graceful_maintenance_set",
            "getval": re.compile(
                r"""
               \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<graceful_maintenance>graceful-maintenance)
                $""", re.VERBOSE,
            ),
            "setval": "graceful-maintenance",
            "compval": "graceful_maintenance.set",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "graceful_maintenance": {
                                    "set": "{{ True if graceful_maintenance is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_graceful_maintenance_activate",
            "getval": re.compile(
                r"""
               \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<graceful_maintenance>graceful-maintenance\sactivate)
                $""", re.VERBOSE,
            ),
            "setval": "graceful-maintenance activate",
            "compval": "graceful_maintenance.activate.set",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "graceful_maintenance": {
                                    "activate": {"set": "{{ True if graceful_maintenance is defined }}"},
                                },
                            },
                        },
                    },
                },
            },
        },

        {
            "name": "neighbor_graceful_maintenance_activate_inheritance_disable",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<graceful_maintenance>activate\sinheritance-disable)
                $""", re.VERBOSE,
            ),
            "setval": "graceful-maintenance activate inheritance-disable",
            "compval": "graceful_maintenance.activate.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "graceful_maintenance": {
                                    "activate": {
                                        "inheritance_disable": "{{ True if graceful_maintenance is defined }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_graceful_maintenance_as_prepends",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<as_prepends>as-prepends\sinheritance-disable)
                $""", re.VERBOSE,
            ),
            "setval": "graceful-maintenance as-prepends inheritance-disable",
            "compval": "graceful_maintenance.as_prepends.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "graceful_maintenance": {
                                    "as_prepends": {
                                        "inheritance_disable": "{{ True if as_prepends is defined }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_graceful_maintenance_local_preference_disable",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<local_preference>local-preference\sinheritance-disable)
                $""", re.VERBOSE,
            ),
            "setval": "graceful-maintenance local-preference inheritance-disable",
            "compval": "graceful_maintenance.local_preference.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "graceful_maintenance": {
                                    "local_preference": {
                                        "inheritance_disable": "{{ True if local_preference is defined }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_graceful_maintenance_local_preference",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<local_preference>local-preference\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "graceful-maintenance local-preference {{ graceful_maintenance.local_preference.value}}",
            "compval": "graceful_maintenance.local_preference.value",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "graceful_maintenance": {
                                    "local_preference": {
                                        "value": "{{ local_preference.split(" ")[1]}}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_graceful_maintenance_as_prepends_value",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<as_prepends>as-prepends\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "graceful-maintenance as-prepends {{ graceful_maintenance.as_prepends.value }}",
            "compval": "graceful_maintenance.as_prepends.value",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "graceful_maintenance": {
                                    "as_prepends": {
                                        "value": "{{ as_prepends.split(" ")[1]}}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_capability_additional_paths_send",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \scapability
                \sadditional-paths
                \s(?P<additional_paths_send>send)
                $""", re.VERBOSE,
            ),
            "setval": "capability additional-paths send",
            "compval": "capability.additional_paths.send.set",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "capability": {
                                    "additional_paths": {
                                        "send": {
                                            "set": "{{ True if additional_paths_send is defined }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_capability_additional_paths_send_disable",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \scapability
                \sadditional-paths
                \s(?P<additional_paths_send>send\sdisable)
                $""", re.VERBOSE,
            ),
            "setval": "capability additional-paths send disable",
            "compval": "capability.additional_paths.send.disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "capability": {
                                    "additional_paths": {
                                        "send": {
                                            "disable": "{{ True if additional_paths_send is defined }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_capability_additional_paths_rcv",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \scapability
                \sadditional-paths
                \s(?P<additional_paths_receive>receive)
                $""", re.VERBOSE,
            ),
            "setval": "capability additional-paths receive",
            "compval": "capability.additional_paths.receive.set",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "capability": {
                                    "additional_paths": {
                                        "receive": {
                                            "set": "{{ True if additional_paths_receive is defined }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_capability_additional_paths_rcv_disable",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \scapability
                \sadditional-paths
                \s(?P<additional_paths_receive_disable>receive\sdisable)
                $""", re.VERBOSE,
            ),
            "setval": "capability additional-paths receive disable",
            "compval": "capability.additional_paths.receive.disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "capability": {
                                    "additional_paths": {
                                        "receive": {
                                            "disable": "{{ True if additional_paths_receive_disable is defined }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_capability_suppress_four_byte_AS",
            "getval": re.compile(
                r"""
               \s+(?P<nbr_address>neighbor\s\S+)
                \scapability
                \ssuppress
                \s(?P<suppress_4_byte_as>4-byte-as)
                $""", re.VERBOSE,
            ),
            "setval": "capability suppress 4-byte-as",
            "compval": "capability.suppress.four_byte_AS.set",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "capability": {
                                    "suppress": {
                                        "four_byte_AS": {
                                            "set": "{{ True if suppress_4_byte_as is defined }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_capability_suppress_all",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \scapability
                \ssuppress
                \s(?P<all>all)
                $""", re.VERBOSE,
            ),
            "setval": "capability suppress all",
            "compval": "capability.suppress.all.set",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "capability": {
                                    "suppress": {
                                        "all": {
                                            "set": "{{ True if all is defined }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_capability_suppress_all_inheritance_disable",
            "getval": re.compile(
                r"""
               \s+(?P<nbr_address>neighbor\s\S+)
                \scapability
                \ssuppress
                \s(?P<all>all\sinheritance-disable)
                $""", re.VERBOSE,
            ),
            "setval": "capability suppress all inheritance-disable",
            "compval": "capability.suppress.all.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "capability": {
                                    "suppress": {
                                        "all": {
                                            "inheritance_disable": "{{ True if all is defined }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },

        {
            "name": "neighbor_log_message_in_value",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \slog
                \smessage
                \s(?P<value>in\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "log message in {{ log.message.in.value}}",
            "compval": "log.log_message.in.value",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "log": {
                                    "log_message": {
                                        "in": {
                                            "value": "{{ value.split(" ")[1] }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_log_message_in_disable",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \slog
                \smessage
                \s(?P<disable>in\sdisable)
                $""", re.VERBOSE,
            ),
            "setval": "log message in disable",
            "compval": "log.log_message.in.disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "log": {
                                    "log_message": {
                                        "in": {
                                            "disable": "{{ True if disable is defined }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_log_message_in_inheritance_disable",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \slog
                \smessage
                \s(?P<disable>in\sinheritance-diable)
                $""", re.VERBOSE,
            ),
            "setval": "log message in inheritance-diable",
            "compval": "log.log_message.in.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "log": {
                                    "log_message": {
                                        "in": {
                                            "inheritance_disable": "{{ True if disable is defined }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_log_message_out_value",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \slog
                \smessage
                \s(?P<value>out\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "log message out {{ log.message.out.value}}",
            "compval": "log.log_message.out.value",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "log": {
                                    "log_message": {
                                        "out": {
                                            "value": "{{ value.split(" ")[1] }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_log_message_out_disable",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \slog
                \smessage
                \s(?P<disable>out\sdisable)
                $""", re.VERBOSE,
            ),
            "setval": "log message out disable",
            "compval": "log.log_message.out.disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "log": {
                                    "log_message": {
                                        "out": {
                                            "disable": "{{ True if disable is defined }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_log_message_out_inheritance_disable",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \slog
                \smessage
                \s(?P<disable>out\sinheritance-diable)
                $""", re.VERBOSE,
            ),
            "setval": "log message out inheritance-diable",
            "compval": "log.log_message.out.inheritance_disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "log": {
                                    "log_message": {
                                        "out": {
                                            "inheritance_disable": "{{ True if disable is defined }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_update_in_filtering_attribute_filter_group",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<attribute_filter_group>attribute-filter\sgroup\s\S+)
                $""", re.VERBOSE,
            ),
            "setval": "update in filtering attribute-filter group {{ update.in.filtering.attribute_filter.group }}",
            "compval": "update.in.filtering.attribute_filter.group",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "update": {
                                    "in": {
                                        "filtering": {
                                            "attribute_filter": {
                                                "group": "{{ attribute_filter_group.split(" ")[2] }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_update_in_filtering_logging_disable",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<logging_disable>logging\sdisable)
                $""", re.VERBOSE,
            ),
            "setval": "update in filtering logging disable",
            "compval": "update.in.filtering.logging.disable",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "update": {
                                    "in": {
                                        "filtering": {
                                            "logging": {
                                                "disable": "{{True if logging_disable is defined }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor_update_in_filtering_message_buffers",
            "getval": re.compile(
                r"""
                 \s+(?P<nbr_address>neighbor\s\S+)
                \s(?P<message_buffers>message\sbuffers\s\d+)
                $""", re.VERBOSE,
            ),
            "setval": "update in filtering message buffers {{ update.in.filtering.message.buffers}}",
            "compval": "update.in.filtering.update_message.buffers",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "update": {
                                    "in": {
                                        "filtering": {
                                            "update_message": {
                                                "buffers": "{{ message_buffers.split(" ")[2] }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "rd_auto",
            "getval": re.compile(
                r"""
                \s+rd(?P<rd_auto>\sauto)
                $""", re.VERBOSE,
            ),
            "setval": "rd auto",
            "compval": "rd.auto",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "rd": {
                            "auto": "{{True if rd_auto is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "timers_keepalive",
            "getval": re.compile(
                r"""
                \s+timers\sbgp\s(?P<timers_keepalive_time>\d+)\s(?P<hold_time>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "timers bgp {{ timers.keepalive_time}} {{ timers.holdtime}}",
            "compval": "timers",
            "result": {
                "vrfs": {
                    '{{ "vrf_" + vrf|d() }}': {
                        "timers": {
                            "keepalive_time": "{{ timers_keepalive_time }}",
                            "holdtime": "{{ hold_time}}",
                        },
                    },
                },
            },
        },

    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: assurance_events_info
short_description: Information module for Assurance Events Info
description:
- This module represents an alias of the module assurance_events_v1_info
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceFamily:
    description:
    - >
      DeviceFamily query parameter. Device family. Please note that multiple families across network device type
      and client type is not allowed. For example, choosing `Routers` along with `Wireless Client` or `Unified AP`
      is not supported. Examples `deviceFamily=Switches and Hubs` (single deviceFamily requested)
      `deviceFamily=Switches and Hubs&deviceFamily=Routers` (multiple deviceFamily requested).
    type: str
  startTime:
    description:
    - >
      StartTime query parameter. Start time from which API queries the data set related to the resource. It must
      be specified in UNIX epochtime in milliseconds. Value is inclusive. If `startTime` is not provided, API will
      default to current time minus 24 hours.
    type: float
  endTime:
    description:
    - >
      EndTime query parameter. End time to which API queries the data set related to the resource. It must be
      specified in UNIX epochtime in milliseconds. Value is inclusive. If `endTime` is not provided, API will
      default to current time.
    type: float
  messageType:
    description:
    - >
      MessageType query parameter. Message type for the event. Examples `messageType=Syslog` (single messageType
      requested) `messageType=Trap&messageType=Syslog` (multiple messageType requested).
    type: str
  severity:
    description:
    - >
      Severity query parameter. Severity of the event between 0 and 6. This is applicable only for events related
      to network devices (other than AP) and `Wired Client` events. | Value | Severity | | ----- | ----------- | |
      0 | Emergency | | 1 | Alert | | 2 | Critical | | 3 | Error | | 4 | Warning | | 5 | Notice | | 6 | Info |
      Examples `severity=0` (single severity requested) `severity=0&severity=1` (multiple severity requested).
    type: float
  siteId:
    description:
    - >
      SiteId query parameter. The UUID of the site. (Ex. `flooruuid`) Examples `?siteId=id1` (single siteId
      requested) `?siteId=id1&siteId=id2&siteId=id3` (multiple siteId requested).
    type: str
  siteHierarchyId:
    description:
    - >
      SiteHierarchyId query parameter. The full hierarchy breakdown of the site tree in id form starting from
      Global site UUID and ending with the specific site UUID. (Ex. `globalUuid/areaUuid/buildingUuid/floorUuid`)
      This field supports wildcard asterisk (`*`) character search support. E.g. `*uuid*, *uuid, uuid*` Examples
      `?siteHierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid `(single siteHierarchyId requested) `?siteHiera
      rchyId=globalUuid/areaUuid/buildingUuid/floorUuid&siteHierarchyId=globalUuid/areaUuid2/buildingUuid2/floorUu
      id2` (multiple siteHierarchyId requested).
    type: str
  networkDeviceName:
    description:
    - >
      NetworkDeviceName query parameter. Network device name. This parameter is applicable for network device
      related families. This field supports wildcard (`*`) character-based search. Ex `*Branch*` or `Branch*` or
      `*Branch` Examples `networkDeviceName=Branch-3-Gateway` (single networkDeviceName requested)
      `networkDeviceName=Branch-3-Gateway&networkDeviceName=Branch-3-Switch` (multiple networkDeviceName
      requested).
    type: str
  networkDeviceId:
    description:
    - >
      NetworkDeviceId query parameter. The list of Network Device Uuids. (Ex.
      `6bef213c-19ca-4170-8375-b694e251101c`) Examples `networkDeviceId=6bef213c-19ca-4170-8375-b694e251101c`
      (single networkDeviceId requested) `networkDeviceId=6bef213c-19ca-4170-8375-
      b694e251101c&networkDeviceId=32219612-819e-4b5e-a96b-cf22aca13dd9&networkDeviceId=2541e9a7-b80d-4955-8aa2-
      79b233318ba0` (multiple networkDeviceId with & separator).
    type: str
  apMac:
    description:
    - >
      ApMac query parameter. MAC address of the access point. This parameter is applicable for `Unified AP` and
      `Wireless Client` events. This field supports wildcard (`*`) character-based search. Ex `*50 0F*` or `50
      0F*` or `*50 0F` Examples `apMac=50 0F 80 0F F7 E0` (single apMac requested) `apMac=50 0F 80 0F F7
      E0&apMac=18 80 90 AB 7E A0` (multiple apMac requested).
    type: str
  clientMac:
    description:
    - >
      ClientMac query parameter. MAC address of the client. This parameter is applicable for `Wired Client` and
      `Wireless Client` events. This field supports wildcard (`*`) character-based search. Ex `*66 2B*` or `66
      2B*` or `*66 2B` Examples `clientMac=66 2B B8 D2 01 56` (single clientMac requested) `clientMac=66 2B B8 D2
      01 56&clientMac=DC A6 32 F5 5A 89` (multiple clientMac requested).
    type: str
  attribute:
    description:
    - >
      Attribute query parameter. The list of attributes that needs to be included in the response. If this
      parameter is not provided, then basic attributes (`id`, `name`, `timestamp`, `details`, `messageType`,
      `siteHierarchyId`, `siteHierarchy`, `deviceFamily`, `networkDeviceId`, `networkDeviceName`,
      `managementIpAddress`) would be part of the response. Examples `attribute=name` (single attribute requested)
      `attribute=name&attribute=networkDeviceName` (multiple attribute requested).
    type: str
  view:
    description:
    - >
      View query parameter. The list of events views. Please refer to `EventViews` for the supported list Examples
      `view=network` (single view requested) `view=network&view=ap` (multiple view requested).
    type: str
  offset:
    description:
    - >
      Offset query parameter. Specifies the starting point within all records returned by the API. It's one based
      offset. The starting value is 1.
    type: float
  limit:
    description:
    - Limit query parameter. Maximum number of records to return.
    type: float
  sortBy:
    description:
    - SortBy query parameter. A field within the response to sort by.
    type: str
  order:
    description:
    - Order query parameter. The sort order of the field ascending or descending.
    type: str
  id:
    description:
    - Id path parameter. Unique identifier for the event.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices GetDetailsOfASingleAssuranceEventV1
  description: Complete reference of the GetDetailsOfASingleAssuranceEventV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-details-of-a-single-assurance-event
- name: Cisco DNA Center documentation for Devices QueryAssuranceEventsV1
  description: Complete reference of the QueryAssuranceEventsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!query-assurance-events
notes:
  - SDK Method used are
    devices.Devices.get_details_of_a_single_assurance_event_v1,
    devices.Devices.query_assurance_events_v1,

  - Paths used are
    get /dna/data/api/v1/assuranceEvents,
    get /dna/data/api/v1/assuranceEvents/{id},
  - It should be noted that this module is an alias of assurance_events_v1_info

"""

EXAMPLES = r"""
- name: Get all Assurance Events Info
  cisco.dnac.assurance_events_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deviceFamily: string
    startTime: 0
    endTime: 0
    messageType: string
    severity: 0
    siteId: string
    siteHierarchyId: string
    networkDeviceName: string
    networkDeviceId: string
    apMac: string
    clientMac: string
    attribute: string
    view: string
    offset: 0
    limit: 0
    sortBy: string
    order: string
  register: result

- name: Get Assurance Events Info by id
  cisco.dnac.assurance_events_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    attribute: string
    view: string
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "oldRadioChannelWidth": "string",
        "clientMac": "string",
        "switchNumber": "string",
        "assocRssi": 0,
        "affectedClients": [
          "string"
        ],
        "isPrivateMac": true,
        "frequency": "string",
        "apRole": "string",
        "replacingDeviceSerialNumber": "string",
        "messageType": "string",
        "failureCategory": "string",
        "apSwitchName": "string",
        "apSwitchId": "string",
        "radioChannelUtilization": "string",
        "mnemonic": "string",
        "radioChannelSlot": 0,
        "details": "string",
        "id": "string",
        "lastApDisconnectReason": "string",
        "networkDeviceName": "string",
        "identifier": "string",
        "reasonDescription": "string",
        "vlanId": "string",
        "udnId": "string",
        "auditSessionId": "string",
        "apMac": "string",
        "deviceFamily": "string",
        "radioNoise": "string",
        "wlcName": "string",
        "apRadioOperationState": "string",
        "name": "string",
        "failureIpAddress": "string",
        "newRadioChannelList": "string",
        "duid": "string",
        "roamType": "string",
        "candidateAPs": [
          {
            "apId": "string",
            "apName": "string",
            "apMac": "string",
            "bssid": "string",
            "rssi": 0
          }
        ],
        "replacedDeviceSerialNumber": "string",
        "oldRadioChannelList": "string",
        "ssid": "string",
        "subReasonDescription": "string",
        "wirelessClientEventEndTime": 0,
        "ipv4": "string",
        "wlcId": "string",
        "ipv6": "string",
        "missingResponseAPs": [
          {
            "apId": "string",
            "apName": "string",
            "apMac": "string",
            "bssid": "string",
            "type": "string",
            "frameType": "string"
          }
        ],
        "timestamp": 0,
        "severity": 0,
        "currentRadioPowerLevel": 0,
        "newRadioChannelWidth": "string",
        "assocSnr": 0,
        "authServerIp": "string",
        "childEvents": [
          {
            "id": "string",
            "name": "string",
            "timestamp": 0,
            "wirelessEventType": 0,
            "details": "string",
            "reasonCode": "string",
            "reasonDescription": "string",
            "subReasonCode": "string",
            "subReasonDescription": "string",
            "resultStatus": "string",
            "failureCategory": "string"
          }
        ],
        "connectedInterfaceName": "string",
        "dhcpServerIp": "string",
        "managementIpAddress": "string",
        "previousRadioPowerLevel": 0,
        "resultStatus": "string",
        "radioInterference": "string",
        "networkDeviceId": "string",
        "siteHierarchy": "string",
        "eventStatus": "string",
        "wirelessClientEventStartTime": 0,
        "siteHierarchyId": "string",
        "udnName": "string",
        "facility": "string",
        "lastApResetType": "string",
        "invalidIeAPs": [
          {
            "apId": "string",
            "apName": "string",
            "apMac": "string",
            "bssid": "string",
            "type": "string",
            "frameType": "string",
            "ies": "string"
          }
        ],
        "username": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: business_sda_hostonboarding_ssid_ippool_v1_info
short_description: Information module for Business Sda Hostonboarding Ssid Ippool V1
description:
- Get all Business Sda Hostonboarding Ssid Ippool V1.
- Get SSID to IP Pool Mapping.
version_added: '4.0.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  vlanName:
    description:
    - VlanName query parameter. VLAN Name.
    type: str
  siteNameHierarchy:
    description:
    - SiteNameHierarchy query parameter. Site Name Heirarchy.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Fabric Wireless GetSSIDToIPPoolMappingV1
  description: Complete reference of the GetSSIDToIPPoolMappingV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-ssid-to-ip-pool-mapping
notes:
  - SDK Method used are
    fabric_wireless.FabricWireless.get_ssid_to_ip_pool_mapping_v1,

  - Paths used are
    get /dna/intent/api/v1/business/sda/hostonboarding/ssid-ippool,

"""

EXAMPLES = r"""
- name: Get all Business Sda Hostonboarding Ssid Ippool V1
  cisco.dnac.business_sda_hostonboarding_ssid_ippool_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    vlanName: string
    siteNameHierarchy: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "vlanName": "string",
      "ssidDetails": [
        {
          "name": "string",
          "scalableGroupName": "string"
        }
      ]
    }
"""



#!/usr/bin/python
#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: asa_config
author: Peter Sprygada (@privateip), Patrick Ogenstad (@ogenstad)
short_description: Manage configuration sections on Cisco ASA devices
description:
- Cisco ASA configurations use a simple block indent file syntax for segmenting configuration
  into sections.  This module provides an implementation for working with ASA configuration
  sections in a deterministic way.
version_added: 1.0.0
extends_documentation_fragment:
- cisco.asa.asa
options:
  lines:
    description:
    - The ordered set of commands that should be configured in the section.  The commands
      must be the exact same commands as found in the device running-config.  Be sure
      to note the configuration command syntax as some commands are automatically
      modified by the device config parser.
    aliases:
    - commands
    type: list
    elements: str
  parents:
    description:
    - The ordered set of parents that uniquely identify the section or hierarchy the
      commands should be checked against.  If the parents argument is omitted, the
      commands are checked against the set of top level or global commands.
    type: list
    elements: str
  src:
    description:
    - Specifies the source path to the file that contains the configuration or configuration
      template to load.  The path to the source file can either be the full path on
      the Ansible control host or a relative path from the playbook or role root directory.  This
      argument is mutually exclusive with I(lines), I(parents).
    type: path
  before:
    description:
    - The ordered set of commands to push on to the command stack if a change needs
      to be made.  This allows the playbook designer the opportunity to perform configuration
      commands prior to pushing any changes without affecting how the set of commands
      are matched against the system.
    type: list
    elements: str
  after:
    description:
    - The ordered set of commands to append to the end of the command stack if a change
      needs to be made.  Just like with I(before) this allows the playbook designer
      to append a set of commands to be executed after the command set.
    type: list
    elements: str
  match:
    description:
    - Instructs the module on the way to perform the matching of the set of commands
      against the current device config.  If match is set to I(line), commands are
      matched line by line.  If match is set to I(strict), command lines are matched
      with respect to position.  If match is set to I(exact), command lines must be
      an equal match.  Finally, if match is set to I(none), the module will not attempt
      to compare the source configuration with the running configuration on the remote
      device.
    default: line
    choices:
    - line
    - strict
    - exact
    - none
    type: str
  replace:
    description:
    - Instructs the module on the way to perform the configuration on the device.  If
      the replace argument is set to I(line) then the modified lines are pushed to
      the device in configuration mode.  If the replace argument is set to I(block)
      then the entire command block is pushed to the device in configuration mode
      if any line is not correct
    default: line
    choices:
    - line
    - block
    type: str
  backup:
    description:
    - This argument will cause the module to create a full backup of the current C(running-config)
      from the remote device before any changes are made. If the C(backup_options)
      value is not given, the backup file is written to the C(backup) folder in the
      playbook root directory. If the directory does not exist, it is created.
    type: bool
    default: false
  config:
    description:
    - The C(config) argument allows the playbook designer to supply the base configuration
      to be used to validate configuration changes necessary.  If this argument is
      provided, the module will not download the running-config from the remote node.
    type: str
  defaults:
    description:
    - This argument specifies whether or not to collect all defaults when getting
      the remote device running config.  When enabled, the module will get the current
      config by issuing the command C(show running-config all).
    type: bool
    default: no
  passwords:
    description:
    - This argument specifies to include passwords in the config when retrieving the
      running-config from the remote device.  This includes passwords related to VPN
      endpoints.  This argument is mutually exclusive with I(defaults).
    type: bool
  save:
    description:
    - The C(save) argument instructs the module to save the running- config to the
      startup-config at the conclusion of the module running.  If check mode is specified,
      this argument is ignored.
    type: bool
    default: no
  backup_options:
    description:
    - This is a dict object containing configurable options related to backup file
      path. The value of this option is read only when C(backup) is set to I(true),
      if C(backup) is set to I(no) this option will be silently ignored.
    suboptions:
      filename:
        description:
        - The filename to be used to store the backup configuration. If the filename
          is not given it will be generated based on the hostname, current time and
          date in format defined by <hostname>_config.<current-date>@<current-time>
        type: str
      dir_path:
        description:
        - This option provides the path ending with directory name in which the backup
          configuration file will be stored. If the directory does not exist it will
          be first created and the filename is either the value of C(filename) or
          default filename as described in C(filename) options description. If the
          path value is not given in that case a I(backup) directory will be created
          in the current working directory and backup configuration will be copied
          in C(filename) within I(backup) directory.
        type: path
    type: dict
  save_when:
    description:
    - When changes are made to the device running-configuration, the changes are not
      copied to non-volatile storage by default.  Using this argument will change
      that before.  If the argument is set to I(always), then the running-config will
      always be copied to the startup-config and the I(modified) flag will always
      be set to True.  If the argument is set to I(modified), then the running-config
      will only be copied to the startup-config if it has changed since the last save
      to startup-config.  If the argument is set to I(never), the running-config will
      never be copied to the startup-config.  If the argument is set to I(changed),
      then the running-config will only be copied to the startup-config if the task
      has made a change. I(changed) was added in Ansible 2.5.
    default: never
    version_added: 1.1.0
    choices:
    - always
    - never
    - modified
    - changed
    type: str
"""

EXAMPLES = """
- cisco.asa.asa_config:
    lines:
      - network-object host 10.80.30.18
      - network-object host 10.80.30.19
      - network-object host 10.80.30.20
    parents: [object-group network OG-MONITORED-SERVERS]

- cisco.asa.asa_config:
    host: '{{ inventory_hostname }}'
    lines:
      - message-length maximum client auto
      - message-length maximum 512
    match: line
    parents: [policy-map type inspect dns PM-DNS, parameters]
    auth_pass: cisco
    username: admin
    password: cisco
    context: ansible

- cisco.asa.asa_config:
    lines:
      - ikev1 pre-shared-key MyS3cretVPNK3y
    parents: tunnel-group 1.1.1.1 ipsec-attributes
    passwords: true

- name: attach ASA acl on interface vlan13/nameif cloud13
  cisco.asa.asa_config:
    lines:
      - access-group cloud-acl_access_in in interface cloud13

- name: configure ASA (>=9.2) default BGP
  cisco.asa.asa_config:
    lines:
      - bgp log-neighbor-changes
      - bgp bestpath compare-routerid
    parents:
      - router bgp 65002
  register: bgp
  when: bgp_default_config is defined
- name: configure ASA (>=9.2) BGP neighbor in default/single context mode
  cisco.asa.asa_config:
    lines:
      - bgp router-id {{ bgp_router_id }}
      - neighbor {{ bgp_neighbor_ip }} remote-as {{ bgp_neighbor_as }}
      - neighbor {{ bgp_neighbor_ip }} description {{ bgp_neighbor_name }}
    parents:
      - router bgp 65002
      - address-family ipv4 unicast
  register: bgp
  when: bgp_neighbor_as is defined
- name: configure ASA interface with standby
  cisco.asa.asa_config:
    lines:
      - description my cloud interface
      - nameif cloud13
      - security-level 50
      - ip address 192.168.13.1 255.255.255.0 standby 192.168.13.2
    parents: [interface Vlan13]
  register: interface
- name: Show changes to interface from task above
  ansible.builtin.debug:
    var: interface

- name: configurable backup path
  cisco.asa.asa_config:
    lines:
      - access-group cloud-acl_access_in in interface cloud13
    backup: true
    backup_options:
      filename: backup.cfg
      dir_path: /home/user

- name: save running to startup when modified
  cisco.asa.asa_config:
    save_when: modified
"""

RETURN = """
updates:
  description: The set of commands that will be pushed to the remote device
  returned: always
  type: list
  sample: ['...', '...']
backup_path:
  description: The full path to the backup file
  returned: when backup is true
  type: str
  sample: /playbooks/ansible/backup/asa_config.2016-07-16@22:28:34
"""
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.config import (
    NetworkConfig,
    dumps,
)

from ansible_collections.cisco.asa.plugins.module_utils.network.asa.asa import (
    asa_argument_spec,
    check_args,
    get_config,
    load_config,
    run_commands,
)


def get_candidate(module):
    candidate = NetworkConfig(indent=1)
    if module.params["src"]:
        candidate.load(module.params["src"])
    elif module.params["lines"]:
        parents = module.params["parents"] or list()
        candidate.add(module.params["lines"], parents=parents)
    return candidate


def save_config(module, result):
    result["changed"] = True
    if not module.check_mode:
        run_commands(module, "write mem")


def run(module, result):
    match = module.params["match"]
    replace = module.params["replace"]
    path = module.params["parents"]

    candidate = get_candidate(module)
    if match != "none":
        contents = module.params["config"]
        if not contents:
            contents = get_config(module)
        config = NetworkConfig(indent=1, contents=contents)
        configobjs = candidate.difference(
            config,
            path=path,
            match=match,
            replace=replace,
        )

    else:
        configobjs = candidate.items

    if configobjs:
        commands = dumps(configobjs, "commands").split("\n")

        if module.params["lines"]:
            if module.params["before"]:
                commands[:0] = module.params["before"]

            if module.params["after"]:
                commands.extend(module.params["after"])

        result["updates"] = commands

        # send the configuration commands to the device and merge
        # them with the current running config
        if not module.check_mode:
            load_config(module, commands)
        result["changed"] = True

    if module.params["save"]:
        module.warn(
            "module param save is deprecated, please use newer and updated param save_when instead which is released with more functionality!",
        )
        save_config(module, result)
    if module.params["save_when"] == "always":
        save_config(module, result)
    elif module.params["save_when"] == "modified":
        running_config_checksum = run_commands(
            module,
            "show running-config | include checksum:",
        )
        startup_config_checksum = run_commands(
            module,
            "show startup-config | include checksum:",
        )
        if running_config_checksum != startup_config_checksum:
            save_config(module, result)
    elif module.params["save_when"] == "changed" and result["changed"]:
        save_config(module, result)


def main():
    """main entry point for module execution"""
    backup_spec = dict(filename=dict(), dir_path=dict(type="path"))
    argument_spec = dict(
        src=dict(type="path"),
        lines=dict(aliases=["commands"], type="list", elements="str"),
        parents=dict(type="list", elements="str"),
        before=dict(type="list", elements="str"),
        after=dict(type="list", elements="str"),
        match=dict(
            default="line",
            choices=["line", "strict", "exact", "none"],
        ),
        replace=dict(default="line", choices=["line", "block"]),
        backup_options=dict(type="dict", options=backup_spec),
        config=dict(),
        defaults=dict(type="bool", default=False),
        passwords=dict(type="bool", default=False),
        backup=dict(type="bool", default=False),
        save=dict(type="bool", default=False),
        save_when=dict(
            choices=["always", "never", "modified", "changed"],
            default="never",
        ),
    )

    argument_spec.update(asa_argument_spec)

    mutually_exclusive = [
        ("lines", "src"),
        ("parents", "src"),
        ("defaults", "passwords"),
    ]

    required_if = [
        ("match", "strict", ["lines"]),
        ("match", "exact", ["lines"]),
        ("replace", "block", ["lines"]),
    ]

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=mutually_exclusive,
        required_if=required_if,
        supports_check_mode=True,
    )

    result = {"changed": False}

    check_args(module)

    if module.params["backup"]:
        result["__backup__"] = get_config(module)

    run(module, result)

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Gaspard Micol (@gmicol) <gmicol@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_node_block
short_description: Manage Node Block (infra:NodeBlk)
description:
- Manage Node Blocks on Cisco ACI fabrics.
- A node block is a range of nodes. Each node block begins with the first port and ends with the last port.
options:
  switch_profile:
    description:
    - The name of the Fabric access policy leaf/spine switch profile.
    type: str
    aliases:
    - leaf_profile_name
    - leaf_profile
    - switch_profile_name
    - spine_switch_profile
    - spine_switch_profile_name
  access_port_selector:
    description:
    -  The name of the Fabric access policy leaf/spine switch port selector.
    type: str
    aliases: [ access_port_selector_name, port_selector, port_selector_name ]
  node_block:
    description:
    - The name of the Node Block.
    type: str
    aliases: [ node_block_name, name ]
  description:
    description:
    - The description for the Node Block.
    type: str
    aliases: [ node_block_description ]
  from_port:
    description:
    - The beginning of the port range block for the Node Block.
    type: str
    aliases: [ from, from_port_range ]
  to_port:
    description:
    - The end of the port range block for the Node Block.
    type: str
    aliases: [ to, to_port_range ]
  type_node:
    description:
    - The type of Node Block to be created under respective access port.
    type: str
    choices: [ leaf, spine ]
    aliases: [ type ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
-  If Adding a port block on an access leaf switch port selector of I(type) C(leaf),
  The I(switch_profile) and I(access_port_selector) must exist before using this module in your playbook.
  The M(cisco.aci.aci_switch_policy_leaf_profile) and M(cisco.aci.aci_switch_leaf_selector) modules can be used for this.
-  If Adding a port block on an access switch port selector of C(type) C(spine),
  The I(switch_profile) and I(access_port_selector) must exist before using this module in your playbook.
  The M(cisco.aci.aci_access_spine_switch_profile) and M(cisco.aci.aci_access_spine_switch_selector) modules can be used for this.
seealso:
- module: cisco.aci.aci_switch_policy_leaf_profile
- module: cisco.aci.aci_switch_leaf_selector
- module: cisco.aci.aci_access_spine_switch_profile
- module: cisco.aci.aci_access_spine_switch_selector
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(infra:NodeBlk).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Gaspard Micol (@gmicol)
"""

EXAMPLES = r"""
- name: Add a new Node Block associated to a switch policy leaf profile selector
  cisco.aci.aci_node_block:
    host: apic
    username: admin
    password: SomeSecretPassword
    switch_profile: my_leaf_switch_profile
    access_port_selector: my_leaf_switch_selector
    node_block: my_node_block
    from_port: 1011
    to_port: 1011
    type_node: leaf
    state: present
  delegate_to: localhost

- name: Add a new Node Block associated to a switch policy spine profile selector
  cisco.aci.aci_node_block:
    host: apic
    username: admin
    password: SomeSecretPassword
    switch_profile: my_spine_switch_profile
    access_port_selector: my_spine_switch_selector
    node_block: my_node_block
    from_port: 1012
    to_port: 1012
    type_node: spine
    state: present
  delegate_to: localhost

- name: Query a Node Block associated to a switch policy leaf profile selector
  cisco.aci.aci_node_block:
    host: apic
    username: admin
    password: SomeSecretPassword
    switch_profile: my_leaf_switch_profile
    access_port_selector: my_leaf_switch_selector
    node_block: my_node_block
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all Node Blocks under the switch policy leaf profile selector
  cisco.aci.aci_node_block:
    host: apic
    username: admin
    password: SomeSecretPassword
    switch_profile: my_leaf_switch_profile
    access_port_selector: my_leaf_switch_selector
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all Node Blocks
  cisco.aci.aci_node_block:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Remove a Node Block associated to a switch policy leaf profile selector
  cisco.aci.aci_node_block:
    host: apic
    username: admin
    password: SomeSecretPassword
    switch_profile: my_leaf_switch_profile
    access_port_selector: my_leaf_switch_selector
    node_block: my_node_block
    type_node: leaf
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        switch_profile=dict(
            type="str",
            aliases=[
                "leaf_profile_name",
                "leaf_profile",
                "switch_profile_name",
                "spine_switch_profile",
                "spine_switch_profile_name",
            ],
        ),  # Not required for querying all objects
        access_port_selector=dict(
            type="str",
            aliases=[
                "access_port_selector_name",
                "port_selector",
                "port_selector_name",
            ],
        ),  # Not required for querying all objects
        node_block=dict(type="str", aliases=["node_block_name", "name"]),  # Not required for querying all objects
        description=dict(type="str", aliases=["node_block_description"]),
        from_port=dict(type="str", aliases=["from", "from_port_range"]),
        to_port=dict(type="str", aliases=["to", "to_port_range"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        type_node=dict(type="str", choices=["leaf", "spine"], aliases=["type"]),  # Not required for querying all objects
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["switch_profile", "access_port_selector", "node_block", "type_node"]],
            ["state", "present", ["switch_profile", "access_port_selector", "node_block", "type_node"]],
        ],
    )

    switch_profile = module.params.get("switch_profile")
    access_port_selector = module.params.get("access_port_selector")
    node_block = module.params.get("node_block")
    description = module.params.get("description")
    from_port = module.params.get("from_port")
    to_port = module.params.get("to_port")
    state = module.params.get("state")
    type_node = module.params.get("type_node")

    aci = ACIModule(module)

    if type_node == "spine":
        subclass_1 = dict(
            aci_class="infraSpineP",
            aci_rn="spprof-{0}".format(switch_profile),
            module_object=switch_profile,
            target_filter={"name": switch_profile},
        )
        subclass_2 = dict(
            aci_class="infraSpineS",
            aci_rn="spines-{0}-typ-range".format(access_port_selector),
            module_object=access_port_selector,
            target_filter={"name": access_port_selector},
        )
    else:
        subclass_1 = dict(
            aci_class="infraNodeP",
            aci_rn="nprof-{0}".format(switch_profile),
            module_object=switch_profile,
            target_filter={"name": switch_profile},
        )
        subclass_2 = dict(
            aci_class="infraLeafS",
            aci_rn="leaves-{0}-typ-range".format(access_port_selector),
            module_object=access_port_selector,
            target_filter={"name": access_port_selector},
        )
    aci.construct_url(
        root_class=dict(
            aci_class="infraInfra",
            aci_rn="infra",
        ),
        subclass_1=subclass_1,
        subclass_2=subclass_2,
        subclass_3=dict(
            aci_class="infraNodeBlk",
            aci_rn="nodeblk-{0}".format(node_block),
            module_object=node_block,
            target_filter={"name": node_block},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="infraNodeBlk",
            class_config=dict(
                descr=description,
                name=node_block,
                from_=from_port,
                to_=to_port,
            ),
        )

        aci.get_diff(aci_class="infraNodeBlk")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The module file for iosxr_lacp_interfaces
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: iosxr_lacp_interfaces
short_description: Resource module to configure LACP interfaces.
description:
- This module manages Link Aggregation Control Protocol (LACP) attributes of interfaces
  on IOS-XR devices.
version_added: 1.0.0
notes:
- This module works with connection C(network_cli). See L(the IOS-XR Platform Options,../network/user_guide/platform_iosxr.html).
author: Nilashish Chakraborty (@nilashishc)
options:
  config:
    description: A dictionary of LACP interfaces options.
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - Name/Identifier of the interface or Ether-Bundle.
        type: str
      churn_logging:
        description:
        - Specifies the parameter for logging of LACP churn events.
        - Valid only for ether-bundles.
        - Mode 'actor' logs actor churn events only.
        - Mode 'partner' logs partner churn events only.
        - Mode 'both' logs actor and partner churn events only.
        type: str
        choices:
        - actor
        - partner
        - both
      collector_max_delay:
        description:
        - Specifies the collector max delay to be signaled to the LACP partner.
        - Valid only for ether-bundles.
        - Refer to vendor documentation for valid values.
        type: int
      period:
        description:
        - Specifies the rate at which packets are sent or received.
        - For ether-bundles, this specifies the period to be used by its member links.
        - Refer to vendor documentation for valid values.
        type: int
      switchover_suppress_flaps:
        description:
        - Specifies the time for which to suppress flaps during a LACP switchover.
        - Valid only for ether-bundles.
        - Refer to vendor documentation for valid values.
        type: int
      system:
        description:
        - This dict object contains configurable options related to LACP system parameters
          for ether-bundles.
        type: dict
        suboptions:
          priority:
            description:
            - Specifies the system priority to use in LACP negotiations for the bundle.
            - Refer to vendor documentation for valid values.
            type: int
          mac:
            description:
            - Specifies the system ID to use in LACP negotiations for the bundle,
              encoded as a MAC address.
            type: str
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the IOS-XR device
      by executing the command B(show running-config int).
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    description:
    - The state of the configuration after module completion.
    type: str
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - parsed
    - gathered
    - rendered
    default: merged

"""
EXAMPLES = """
# Using merged
#
#
# ------------
# Before state
# ------------
#
#
#
# RP/0/0/CPU0:an-iosxr#sh running-config interface
# Sun Jul 21 18:01:35.079 UTC
# interface Bundle-Ether10
# !
# interface Bundle-Ether11
# !
# interface Bundle-Ether12
# !
# interface Loopback888
#  description test for ansible
#  shutdown
# !
# interface MgmtEth0/0/CPU0/0
#  ipv4 address 192.0.2.11 255.255.255.0
# !
# interface GigabitEthernet0/0/0/1
#  description 'GigabitEthernet - 1'
# !
# interface GigabitEthernet0/0/0/2
#  description "GigabitEthernet - 2"
# !
# interface GigabitEthernet0/0/0/3
#  description "GigabitEthernet - 3"
# !
# interface GigabitEthernet0/0/0/4
#  description "GigabitEthernet - 4"
# !
#
#

- name: Merge provided configuration with device configuration
  cisco.iosxr.iosxr_lacp_interfaces:
    config:
      - name: Bundle-Ether10
        churn_logging: actor
        collector_max_delay: 100
        switchover_suppress_flaps: 500
      - name: Bundle-Ether11
        system:
          mac: 00c2.4c00.bd15
      - name: GigabitEthernet0/0/0/1
        period: 200
    state: merged


#
#
# -----------
# After state
# -----------
#
#
# RP/0/0/CPU0:an-iosxr#sh run int
# Sun Jul 21 18:24:52.413 UTC
# interface Bundle-Ether10
#  lacp churn logging actor
#  lacp switchover suppress-flaps 500
#  lacp collector-max-delay 100
# !
# interface Bundle-Ether11
#  lacp system mac 00c2.4c00.bd15
# !
# interface Bundle-Ether12
# !
# interface Loopback888
#  description test for ansible
#  shutdown
# !
# interface MgmtEth0/0/CPU0/0
#  ipv4 address 192.0.2.11 255.255.255.0
# !
# interface GigabitEthernet0/0/0/1
#  description 'GigabitEthernet - 1"
#  lacp period 200
# !
# interface GigabitEthernet0/0/0/2
#  description "GigabitEthernet - 2"
# !
# interface GigabitEthernet0/0/0/3
#  description "GigabitEthernet - 3"
# !
# interface GigabitEthernet0/0/0/4
#  description "GigabitEthernet - 4"
# !
#


# Using replaced
#
#
# ------------
# Before state
# ------------
#
#
# RP/0/0/CPU0:an-iosxr#sh run int
# Sun Jul 21 18:24:52.413 UTC
# interface Bundle-Ether10
#  lacp churn logging actor
#  lacp switchover suppress-flaps 500
#  lacp collector-max-delay 100
# !
# interface Bundle-Ether11
#  lacp system mac 00c2.4c00.bd15
# !
# interface Bundle-Ether12
# !
# interface Loopback888
#  description test for ansible
#  shutdown
# !
# interface MgmtEth0/0/CPU0/0
#  ipv4 address 192.0.2.11 255.255.255.0
# !
# interface GigabitEthernet0/0/0/1
#  description 'GigabitEthernet - 1"
#  lacp period 200
# !
# interface GigabitEthernet0/0/0/2
#  description "GigabitEthernet - 2"
# !
# interface GigabitEthernet0/0/0/3
#  description "GigabitEthernet - 3"
# !
# interface GigabitEthernet0/0/0/4
#  description "GigabitEthernet - 4"
# !
#

- name: Replace LACP configuration of listed interfaces with provided configuration
  cisco.iosxr.iosxr_lacp_interfaces:
    config:
      - name: Bundle-Ether10
        churn_logging: partner
      - name: GigabitEthernet0/0/0/2
        period: 300
    state: replaced


#
#
# -----------
# After state
# -----------
#
#
# RP/0/0/CPU0:an-iosxr#sh run int
# Sun Jul 21 18:50:21.929 UTC
# interface Bundle-Ether10
#  lacp churn logging partner
# !
# interface Bundle-Ether11
#  lacp system mac 00c2.4c00.bd15
# !
# interface Bundle-Ether12
# !
# interface Loopback888
#  description test for ansible
#  shutdown
# !
# interface MgmtEth0/0/CPU0/0
#  ipv4 address 192.0.2.11 255.255.255.0
# !
# interface GigabitEthernet0/0/0/1
#  description 'GigabitEthernet - 1"
#  lacp period 200
# !
# interface GigabitEthernet0/0/0/2
#  description "GigabitEthernet - 2"
#  lacp period 300
# !
# interface GigabitEthernet0/0/0/3
#  description "GigabitEthernet - 3"
# !
# interface GigabitEthernet0/0/0/4
#  description "GigabitEthernet - 4"
# !
#
#


# Using overridden
#
#
# ------------
# Before state
# ------------
#
#
# RP/0/0/CPU0:an-iosxr#sh run int
# Sun Jul 21 18:24:52.413 UTC
# interface Bundle-Ether10
#  lacp churn logging actor
#  lacp switchover suppress-flaps 500
#  lacp collector-max-delay 100
# !
# interface Bundle-Ether11
#  lacp system mac 00c2.4c00.bd15
# !
# interface Bundle-Ether12
# !
# interface Loopback888
#  description test for ansible
#  shutdown
# !
# interface MgmtEth0/0/CPU0/0
#  ipv4 address 192.0.2.11 255.255.255.0
# !
# interface GigabitEthernet0/0/0/1
#  description 'GigabitEthernet - 1"
#  lacp period 200
# !
# interface GigabitEthernet0/0/0/2
#  description "GigabitEthernet - 2"
#  lacp period 200
# !
# interface GigabitEthernet0/0/0/3
#  description "GigabitEthernet - 3"
# !
# interface GigabitEthernet0/0/0/4
#  description "GigabitEthernet - 4"
# !
#
#

- name: Override all interface LACP configuration with provided configuration
  cisco.iosxr.iosxr_lacp_interfaces:
    config:
      - name: Bundle-Ether12
        churn_logging: both
        collector_max_delay: 100
        switchover_suppress_flaps: 500
      - name: GigabitEthernet0/0/0/1
        period: 300
    state: overridden


#
#
# -----------
# After state
# -----------
#
#
# RP/0/0/CPU0:an-iosxr(config-if)#do sh run int
# Sun Jul 21 19:32:36.115 UTC
# interface Bundle-Ether10
# !
# interface Bundle-Ether11
# !
# interface Bundle-Ether12
#  lacp churn logging both
#  lacp switchover suppress-flaps 500
#  lacp collector-max-delay 100
# !
# interface Loopback888
#  description test for ansible
#  shutdown
# !
# interface MgmtEth0/0/CPU0/0
#  ipv4 address 192.0.2.11 255.255.255.0
# !
# interface GigabitEthernet0/0/0/1
#  description 'GigabitEthernet - 1"
#  lacp period 300
# !
# interface GigabitEthernet0/0/0/2
#  description "GigabitEthernet - 2"
# !
# interface GigabitEthernet0/0/0/3
#  description "GigabitEthernet - 3"
# !
# interface GigabitEthernet0/0/0/4
#  description "GigabitEthernet - 4"
# !
#


# Using deleted
#
#
# ------------
# Before state
# ------------
#
#
# RP/0/0/CPU0:an-iosxr#sh run int
# Sun Jul 21 18:24:52.413 UTC
# interface Bundle-Ether10
#  lacp churn logging actor
#  lacp switchover suppress-flaps 500
#  lacp collector-max-delay 100
# !
# interface Bundle-Ether11
#  lacp non-revertive
# !
# interface Bundle-Ether12
# !
# interface Loopback888
#  description test for ansible
#  shutdown
# !
# interface MgmtEth0/0/CPU0/0
#  ipv4 address 192.0.2.11 255.255.255.0
# !
# interface GigabitEthernet0/0/0/1
#  description 'GigabitEthernet - 1"
#  lacp period 200
# !
# interface GigabitEthernet0/0/0/2
#  description "GigabitEthernet - 2"
#   lacp period 300
# !
# interface GigabitEthernet0/0/0/3
#  description "GigabitEthernet - 3"
# !
# interface GigabitEthernet0/0/0/4
#  description "GigabitEthernet - 4"
# !
#

- name: >-
    Deleted LACP configurations of provided interfaces (Note - This won't delete
    the interface itself)
  cisco.iosxr.iosxr_lacp_interfaces:
    config:
      - name: Bundle-Ether10
      - name: Bundle-Ether11
      - name: GigabitEthernet0/0/0/1
      - name: GigabitEthernet0/0/0/2
    state: deleted


#
#
# -----------
# After state
# -----------
#
#
# Using parsed:

# parsed.cfg
# interface Bundle-Ether10
#  lacp churn logging actor
#  lacp switchover suppress-flaps 500
#  lacp collector-max-delay 100
# !
# interface Bundle-Ether11
#  lacp system mac 00c2.4c00.bd15
# !
# interface MgmtEth0/0/CPU0/0
#  ipv4 address 192.0.2.11 255.255.255.0
# !
# interface GigabitEthernet0/0/0/1
#  lacp period 200
# !
#

- name: Convert lacp interfaces config to argspec without connecting to the appliance
  cisco.iosxr.iosxr_lacp_interfaces:
    running_config: "{{ lookup('file', './parsed.cfg') }}"
    state: parsed

# --------------
# Output:
# --------------

#    parsed:
#      - name: Bundle-Ether10
#        churn_logging: actor
#        collector_max_delay: 100
#        switchover_suppress_flaps: 500
#
#      - name: Bundle-Ether11
#        system:
#          mac: 00c2.4c00.bd15
#
#      - name: GigabitEthernet0/0/0/1
#        period: 200
#
#

# Using gathered:

# Native config:
# interface Bundle-Ether10
#  lacp churn logging actor
#  lacp switchover suppress-flaps 500
#  lacp collector-max-delay 100
# !
# interface Bundle-Ether11
#  lacp system mac 00c2.4c00.bd15
# !
# interface MgmtEth0/0/CPU0/0
#  ipv4 address 192.0.2.11 255.255.255.0
# !
# interface GigabitEthernet0/0/0/1
#  lacp period 200
# !
#

- name: Gather IOSXR lacp interfaces configuration
  cisco.iosxr.iosxr_lacp_interfaces:
    config:
    state: gathered

# ----------
# Output
# ---------
#    gathered:
#      - name: Bundle-Ether10
#        churn_logging: actor
#        collector_max_delay: 100
#        switchover_suppress_flaps: 500
#
#      - name: Bundle-Ether11
#        system:
#          mac: 00c2.4c00.bd15
#
#      - name: GigabitEthernet0/0/0/1
#        period: 200

# Using rendered:

- name: Render platform specific commands from task input using rendered state
  cisco.iosxr.iosxr_lacp_interfaces:
    config:
      - name: Bundle-Ether10
        churn_logging: actor
        collector_max_delay: 100
        switchover_suppress_flaps: 500
      - name: Bundle-Ether11
        system:
          mac: 00c2.4c00.bd15
      - name: GigabitEthernet0/0/0/1
        period: 200
    state: rendered


# -------------
# Output
# -------------
# rendered: [
#     - "interface Bundle-Ether10"
#     - " lacp churn logging actor"
#     - " lacp switchover suppress-flaps 500"
#     - " lacp collector-max-delay 100"
#     - "interface Bundle-Ether11"
#     - " lacp system mac 00c2.4c00.bd15"
#     - "interface MgmtEth0/0/CPU0/0"
#     - " ipv4 address 192.0.2.11 255.255.255.0"
#     - "interface GigabitEthernet0/0/0/1"
#     - " lacp period 200"
#
"""
RETURN = """
before:
  description: The configuration as structured data prior to module invocation.
  returned: always
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The configuration as structured data after module completion.
  returned: when changed
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample: ['interface Bundle-Ether10', 'lacp churn logging partner', 'lacp period 150']
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.lacp_interfaces.lacp_interfaces import (
    Lacp_interfacesArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.lacp_interfaces.lacp_interfaces import (
    Lacp_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    required_if = [
        ("state", "merged", ("config",)),
        ("state", "replaced", ("config",)),
        ("state", "overridden", ("config",)),
        ("state", "rendered", ("config",)),
        ("state", "parsed", ("running_config",)),
    ]

    mutually_exclusive = [("config", "running_config")]
    module = AnsibleModule(
        argument_spec=Lacp_interfacesArgs.argument_spec,
        required_if=required_if,
        supports_check_mode=True,
        mutually_exclusive=mutually_exclusive,
    )

    result = Lacp_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()
